!******************************************************************************!
!                                                                              !
!    lcFaVF1.f90                                                               !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcFaVF1.f90                                    |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2016                                 |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2018-10-11 09:04             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcFaVF1
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29, x30, x31, x32, x33, x34, x35, x36, x37,  &
                  x38, x39, x40, x41, x42, x43, x44, x45, x46
  complex (dp), dimension(0:3) :: x0vec, x1vec, x2vec, x3vec, x4vec, x5vec,    &
                                  x6vec, x7vec
  contains
    subroutine loopFaVF1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,     &
    riMaxOut, rankInc)
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(FaVF1)
!       GammaP(2,1,3)*c0
!       GammaM(2,1,3)*c1
        pl(:) = PL1(:) + PL2(:)
        x0 = pl(2)*w2(2)
        x1 = cima*w2(1)
        x2 = pl(2)*x1
        x3 = cima*w2(0)
        x4 = cima*w2(3)
        x5 = pl(1)*x3 + pl(1)*x4
        x7 = pl(1)*w2(2)
        x8 = pl(1)*x1
        x9 = pl(2)*x4
        x10 = pl(2)*x3
        x12 = -w2(2) + x1
        x14 = co(0)*m
        x15 = w2(0) - w2(3)
        x18 = pl(0)*w2(2)
        x19 = pl(0)*x1
        x20 = -pl(3)*x3
        x21 = pl(3)*x4
        x22 = pl(0)*x3
        x23 = pl(0)*x4
        x24 = pl(3)*w2(2) - pl(3)*x1
        x25 = w2(2) + x1
        x27 = w2(0) + w2(3)
        x33 = co(1)*m
        x44 = cima*w2(2)
        x45 = -w2(1) + x44
        x46 = w2(1) + x44
        do j = riMaxIn, 0, -1
          x6 = co(1)*wl(0,j)
          x11 = co(1)*wl(1,j)
          x13 = wl(3,j)*x12
          x16 = wl(2,j)*x15
          x17 = cima*x16
          x26 = wl(2,j)*x25
          x28 = wl(3,j)*x27
          x29 = cima*x28
          x30 = co(0)*wl(2,j)
          x31 = co(0)*wl(3,j)
          x32 = wl(1,j)*x12
          x34 = wl(0,j)*x27
          x35 = cima*x34
          x36 = wl(0,j)*x25
          x37 = wl(1,j)*x15
          x38 = cima*x37
          x39 = -co(1)*(x32 + x35)
          x40 = co(1)*(x36 + x38)
          x41 = -x40
          x42 = co(0)*(x26 - x29)
          x43 = co(0)*(-x13 + x17)
          wp(0,j) = -x11*(-x10 - x7 + x8 + x9) + x13*x14 - x14*x17 - x6*(-x0 - &
                    x2 + x5)
          wp(1,j) = -x11*(x22 - x23 + x24) + x14*x26 - x14*x29 - x6*(x18 + x19 &
                     + x20 - x21)
          wp(2,j) = x30*(x0 + x2 - x22 + x23) - x31*(x10 + x18 - x19 + x9) -   &
                    x32*x33 - x33*x35
          wp(3,j) = x30*(x20 + x21 + x7 + x8) - x31*(x24 + x5) - x33*x36 - x33*&
                    x38
          if(rankInc .lt. 1) cycle
          sp2(0,0) = x39
          sp2(0,1) = x41
          sp2(0,2) = co(0)*(x13 - x17)
          sp2(0,3) = x42
          sp2(1,0) = x41
          sp2(1,1) = x39
          sp2(1,2) = co(0)*(-x26 + x29)
          sp2(1,3) = x43
          sp2(2,0) = co(1)*(-wl(0,j)*x45 + x37)
          sp2(2,1) = -co(1)*(wl(1,j)*x46 + x34)
          sp2(2,2) = -co(0)*(wl(2,j)*x45 + x28)
          sp2(2,3) = co(0)*(-wl(3,j)*x46 + x16)
          sp2(3,0) = x39
          sp2(3,1) = x40
          sp2(3,2) = x43
          sp2(3,3) = x42
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      end select 
    end subroutine loopFaVF1
end module lcFaVF1
